SystemOrganization addCategory: #Everett!
SystemOrganization addCategory: #'Everett-Pong'!
SystemOrganization addCategory: #'Everett-Tests'!
SystemOrganization addCategory: #'Everett-TicTacToe'!

TestCase subclass: #EntityManagerTestCase
	instanceVariableNames: 'em'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

!EntityManagerTestCase methodsFor: 'running' stamp: 'EricShahkarami 7/2/2013 11:47'!
setUp 

	em := EntityManager new.! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 18:50'!
testEntityMatches

	| e1 e2 pc1 bc1 bc2 c1 c2 c3 s1 s2 |
	
	e1 := em createEntity.
	e2 := em createEntity.

	pc1 := PositionComponent position: 3@7.
	bc1 := VelocityComponent velocity: 1@1.
	bc2 := VelocityComponent velocity: 2@0.
	
	em addComponent: pc1 toEntity: e1.
	em addComponent: bc1 toEntity: e1.
	em addComponent: bc2 toEntity: e2.
	
	c1 := Set newFrom: #(#position).
	c2 := Set newFrom: #(#velocity).
	c3 := Set newFrom: #(#position #velocity).

	s1 := Set new.
	s2 := Set new.
	s1 add: e1.
	s2 add: e1.
	s2 add: e2.
	
	self assert: ( (em entitiesContainingComponents: c1) = s1 ).
	self assert: ( (em entitiesContainingComponents: c2) = s2 ).
	self assert: ( (em entitiesContainingComponents: c3) = s1 )! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 23:57'!
testRemoveComponent

	| e pc vc c1 c2 s |
	
	e := em createEntity.

	pc := PositionComponent position: 3@7.
	vc := VelocityComponent velocity: 1@1.
	
	em addComponent: pc toEntity: e.
	em addComponent: vc toEntity: e.
	em removeComponent: (PositionComponent identity) fromEntity: e.
		
	c1 := Set newFrom: #(#velocity).
	c2:= Set newFrom: #(#position velocity).
	
	s := Set new.
	s add: e.
	
	self assert: ( (em entitiesContainingComponents: c1) = s ).
	self assert: ( (em entitiesContainingComponents: c2) isEmpty = true ).! !

TestCase subclass: #GroupManagerTestCase
	instanceVariableNames: 'em e1 e2 e3 pc1 bc1 bc2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!GroupManagerTestCase methodsFor: 'running' stamp: 'ShruthiSudhanva 7/12/2013 13:37'!
setUp 

	em := EntityManager new.
	e1 := em createEntity.
	e2 := em createEntity.
	e3 := em createEntity.
	
	pc1 := PositionComponent position: 3@7.
	bc1 := VelocityComponent velocity: 1@1.
	bc2 := VelocityComponent velocity: 2@0.! !

!GroupManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 19:50'!
testAddComponent

	| gm entities |
	
	gm := em groupManager.
	
	em addComponent: pc1 toEntity: e2.
	gm createGroup: #group1 withEntities: (Set newFrom: {e1 . e2}).
	gm createGroup: #group2 withEntities: (Set newFrom: {e1 . e3 }).
	
	gm addComponent: bc2 toGroup: #group1.
	gm addComponent: bc1 toGroup: #group2.
	gm addEntity: e2 toGroup: #group2.

	self assert: (gm entitiesInGroup: #group1) = (Set newFrom:{0 .1}).
	entities := gm entitiesInGroup: #group2 select: [:entity| (em doesEntity:entity containComponents: (Set newFrom: { #position })) = true ].
	self assert: entities = (Set newFrom: { 1 }).
	
	
	
	
! !

!GroupManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 19:50'!
testRemoveComponent
	
	| gm entities |
	
	gm := em groupManager.
	
	em addComponent: pc1 toEntity: e2.
	gm createGroup: #group1 withEntities: (Set newFrom: {e1 . e2}).
	gm createGroup: #group2 withEntities: (Set newFrom: {e1 . e3 }).
	
	gm addComponent: bc2 toGroup: #group1.
	gm addComponent: bc1 toGroup: #group2.
	gm removeComponent: (bc2 class identity) fromGroup: #group1.

	entities := gm entitiesInGroup: #group1 select: [ :entity | (em doesEntity: entity containComponents: (Set newFrom: { #velocity } )) = true ].
	self assert: entities isEmpty.
	
	gm removeEntity: 0 fromGroup: #group2.
	entities := gm entitiesInGroup: #group2.
	
	self assert: entities = (Set newFrom:{2}).
	
	gm removeGroup: #group1.
	self assert: (gm groups includes: #group1) = false .
	! !

TestCase subclass: #SystemManagerTestCase
	instanceVariableNames: 'em sm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:51'!
setUp

	em := EntityManager new.
	sm := SystemManager new! !

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/7/2013 15:21'!
testSystemUpdate

	"Tests that e1 and e2 have their positions updated but not e3, which doesn't have a velocity component"

	| e1 e2 e3 pc1 pc2 pc3 vc1 vc2 |
	
	e1 := em createEntity.
	e2 := em createEntity.
	e3 := em createEntity.

	pc1 := PositionComponent position: 3@4.
	pc2 := PositionComponent position: 1@1.
	pc3 := PositionComponent position: 9@9.
	vc1 := VelocityComponent velocity: 1@1.
	vc2 := VelocityComponent velocity: 2@0.

	em addComponent: pc1 toEntity: e1.
	em addComponent: pc2 toEntity: e2.
	em addComponent: pc3 toEntity: e3.
	em addComponent: vc1 toEntity: e1.
	em addComponent: vc2 toEntity: e2.
	
	sm addSystem: (MovementSystem new).
	sm updateEntitiesOf: em afterDeltaTime: 1.
	
	self assert: ( (pc1 position) = (4@5) ).
	self assert: ( (pc2 position) = (3@1) ).
	self assert: ( (pc3 position) = (9@9) )! !

SimpleButtonMorph subclass: #CellMorph
	instanceVariableNames: 'wasClicked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!CellMorph methodsFor: 'button' stamp: 'eshkrm 7/23/2013 15:28'!
doButtonAction 

	wasClicked := true! !

!CellMorph methodsFor: 'initialize-release' stamp: 'eshkrm 7/9/2013 16:25'!
initialize 

	super initialize.
	self label: ''.
	self borderWidth: 2.
	bounds := 0@0 corner: 64@64.
	self useSquareCorners.
	wasClicked := false
	! !

!CellMorph methodsFor: 'event handling' stamp: 'eshkrm 7/23/2013 15:27'!
mouseDown: anEvent

	oldColor := self fillStyle. 
	actWhen == #buttonDown ifTrue: [ self doButtonAction ]! !

!CellMorph methodsFor: 'accessing' stamp: 'eshkrm 7/9/2013 21:08'!
wasClicked

	^wasClicked ! !

Object subclass: #EntityFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityFactory commentStamp: '<historical>' prior: 0!
I am a factory that creates entities with some specified component instances. Subclass me to specify which components.!

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/8/2013 14:34'!
createEntity: anEntityManager 

	| entity components |
	
	entity := anEntityManager createEntity.
	components := self obtainComponents.
	components do: [ :component | anEntityManager addComponent: component toEntity: entity ].
	^entity! !

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/11/2013 19:04'!
createInactiveEntity: anEntityManager

	| entity |
	
	entity := self createEntity: anEntityManager.
	anEntityManager deactivateEntity: entity.
	^entity! !

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/8/2013 14:40'!
obtainComponents
	
	^self subclassResponsibility ! !

Object subclass: #EntityManager
	instanceVariableNames: 'entityTable nextAvailableEntity changedEntities inactiveEntities removedEntities groupManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityManager commentStamp: '<historical>' prior: 0!
I am responsible for creating, removing and tracking all entities and their components.
!

!EntityManager methodsFor: 'entity-assign' stamp: 'eshkrm 7/7/2013 13:57'!
activateEntity: anEntity 

	inactiveEntities remove: anEntity.
	self changeEntity: anEntity! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:25'!
addComponent: aComponentInstance toEntity: anEntity

	(anEntity < nextAvailableEntity and: anEntity >= 0) ifTrue: 
	[  
		self entityAt: anEntity putComponent: aComponentInstance.
		self changeEntity: anEntity
	]! !

!EntityManager methodsFor: 'entity-assign' stamp: 'eshkrm 7/6/2013 23:54'!
changeEntity: anEntity

	((changedEntities occurrencesOf: anEntity) > 1) ifFalse: [ changedEntities add: anEntity ]! !

!EntityManager methodsFor: 'entity-enumerate' stamp: 'eshkrm 7/6/2013 19:34'!
changedEntitiesDo: aBlock

	(changedEntities asSet) do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:39'!
clearChangedEntities

	self changedEntitiesDo: [ :entity | changedEntities remove: entity ]! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:39'!
clearRemovedEntities

	self removedEntitiesDo: [ :entity | removedEntities remove: entity ]! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 09:38'!
component: aComponentIdentifier fromEntity: anEntity

	| componentsOfEntity |
	
	componentsOfEntity := self entityAt: anEntity.
	^(componentsOfEntity at: aComponentIdentifier)! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:27'!
createEntity

	entityTable at: nextAvailableEntity put: (Dictionary new).	
	self changeEntity: nextAvailableEntity.
	nextAvailableEntity := nextAvailableEntity + 1.
	^(nextAvailableEntity - 1)
! !

!EntityManager methodsFor: 'entity-assign' stamp: 'eshkrm 7/7/2013 13:49'!
deactivateEntity: anEntity

	inactiveEntities add: anEntity! !

!EntityManager methodsFor: 'entity-testing' stamp: 'eshkrm 6/13/2013 19:41'!
doesEntity: anEntity containComponent: aComponentIdentifier

	| entityComponents |
	
	entityComponents := (self entityAt: anEntity) keys asSet.
	^(entityComponents includes: aComponentIdentifier)! !

!EntityManager methodsFor: 'entity-testing' stamp: 'eshkrm 6/13/2013 18:34'!
doesEntity: anEntity containComponents: aSetOfComponentIdentifiers

	| entityComponents |
	
	entityComponents := (self entityAt: anEntity) keys asSet.
	^( (aSetOfComponentIdentifiers intersection: entityComponents) = aSetOfComponentIdentifiers )! !

!EntityManager methodsFor: 'accessing' stamp: 'eshkrm 7/6/2013 18:50'!
entitiesContainingComponents: aSetOfComponents

	^( self entitiesSelect: [ :entity | self doesEntity: entity containComponents: aSetOfComponents ] )! !

!EntityManager methodsFor: 'entity-enumerate' stamp: 'eshkrm 7/6/2013 23:51'!
entitiesDo: aBlock

	entityTable keysDo: [ :entity | aBlock value: entity ].! !

!EntityManager methodsFor: 'entity-enumerate' stamp: 'EricShahkarami 7/2/2013 11:22'!
entitiesSelect: aBlock

	| selectEntities |
	
	selectEntities := Set new.
	entityTable keysDo: [ :entity | (aBlock value: entity) ifTrue: [ selectEntities add: entity ]  ].
	^selectEntities! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity
	
	^(entityTable at: anEntity)! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity putComponent: aComponentInstance 
		
	(entityTable at: anEntity) at: (aComponentInstance class identity) put: aComponentInstance. ! !

!EntityManager methodsFor: 'accessing' stamp: 'ShruthiSudhanva 7/8/2013 17:07'!
groupManager

	^groupManager.! !

!EntityManager methodsFor: 'entity-enumerate' stamp: 'eshkrm 7/7/2013 13:53'!
inactiveEntitiesDo: aBlock 

	inactiveEntities do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 19:49'!
initialize 
	
	super initialize.
	entityTable := Dictionary new.
	nextAvailableEntity := 0.
	changedEntities := Bag new.
	inactiveEntities := Set new.
	removedEntities := Set new.
	groupManager := GroupManager entityManager: self.! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:25'!
removeComponent: aComponentIdentifier fromEntity: anEntity

	| components |
	
	components := self entityAt: anEntity.
	components removeKey: aComponentIdentifier.
	self changeEntity: anEntity
! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/11/2013 19:59'!
removeEntity: anEntity	

	"Removes entity from entityTable and places entity in removed pile."	
	
	entityTable removeKey: anEntity.
	removedEntities add: anEntity
	! !

!EntityManager methodsFor: 'entity-enumerate' stamp: 'eshkrm 7/6/2013 19:15'!
removedEntitiesDo: aBlock 

	removedEntities do: [ :entity | aBlock value: entity ]! !

Object subclass: #EverettComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettComponent commentStamp: '<historical>' prior: 0!
I hold data for entities. Systems modify me.!

EverettComponent subclass: #BallMorphComponent
	instanceVariableNames: 'canvas morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!BallMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/24/2013 18:46'!
addMorphToCanvas: aWindow

	^(self new addMorphToCanvas: aWindow)! !

!BallMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 6/13/2013 18:29'!
identity

	^#ball! !

!BallMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 18:47'!
addMorphToCanvas: aCanvas

	canvas := aCanvas.
	canvas addMorph: morph.! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:54'!
color

	^(morph color)! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:54'!
color: aColor

	morph color: aColor! !

!BallMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 18:48'!
initialize 

	super initialize.
	morph := CircleMorph new.
	morph color: (Color white).! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:54'!
position

	^(morph position)! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:55'!
position: aPosition

	morph position: aPosition! !

EverettComponent subclass: #BatMorphComponent
	instanceVariableNames: 'morph canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!BatMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/24/2013 18:46'!
addMorphToCanvas: aWindow 

	^(self new addMorphToCanvas: aWindow) ! !

!BatMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 6/13/2013 18:29'!
identity 

	^#bat! !

!BatMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 18:51'!
addMorphToCanvas: aCanvas

	canvas  := aCanvas.
	canvas addMorph: morph! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/19/2013 13:11'!
bottom

	^(morph bottom)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/17/2013 13:30'!
color

	^(morph color)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/17/2013 13:30'!
color: aColor

	morph color: aColor! !

!BatMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 18:49'!
initialize 

	super initialize.
	morph := RectangleMorph new.
	morph width: 15.
	morph height: 75.
	morph color: (Color white).! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/19/2013 13:11'!
left

	^(morph left)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 21:00'!
position

	^(morph position)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 21:01'!
position: aPosition 

	morph position: aPosition ! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/19/2013 13:11'!
right

	^(morph right)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/19/2013 13:11'!
top

	^(morph top)! !

EverettComponent subclass: #BlueComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!BlueComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/23/2013 17:51'!
identity 

	^#blue! !

!BlueComponent methodsFor: 'accessing' stamp: 'ShruthiSudhanva 7/17/2013 21:05'!
color
	^ Color blue.! !

EverettComponent subclass: #BoardIndexComponent
	instanceVariableNames: 'boardIndex boardindex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!BoardIndexComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/23/2013 17:51'!
identity

	^#boardIndex! !

!BoardIndexComponent methodsFor: 'accessing' stamp: 'ShruthiSudhanva 7/17/2013 17:20'!
boardIndex
	
	^boardIndex! !

!BoardIndexComponent methodsFor: 'accessing' stamp: 'eshkrm 7/19/2013 15:56'!
boardIndex: anIndex

	boardIndex := anIndex.! !

EverettComponent subclass: #CellMorphComponent
	instanceVariableNames: 'morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!CellMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 17:54'!
identity

	^#cell! !

!CellMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 15:20'!
position: aPosition 

	^(self new position: aPosition)! !

!CellMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 15:00'!
color: aColor

	morph color: aColor! !

!CellMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/23/2013 15:31'!
height

	^(morph height)! !

!CellMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/16/2013 15:25'!
initialize

	morph := CellMorph new.
! !

!CellMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 14:59'!
morph

	^morph! !

!CellMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 14:59'!
position: aPosition

	morph position: aPosition! !

!CellMorphComponent methodsFor: 'testing' stamp: 'eshkrm 7/16/2013 14:59'!
wasClicked

	^(morph wasClicked)! !

!CellMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/23/2013 15:31'!
width

	^(morph width)! !

!EverettComponent class methodsFor: 'instance creation' stamp: 'eshkrm 7/14/2013 15:21'!
identity

	^self subclassResponsibility! !

EverettComponent subclass: #NewlyCapturedComponent
	instanceVariableNames: 'entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!NewlyCapturedComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/23/2013 17:51'!
identity

 	^#newlyAdded! !

EverettComponent subclass: #PositionComponent
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!PositionComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the position of an entity.!

!PositionComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:09'!
identity
	^#position! !

!PositionComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
position: aPosition

	^(self new position: aPosition)! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
position

	^position! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:02'!
position: aPosition 
	
	position := aPosition! !

EverettComponent subclass: #RedComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!RedComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/23/2013 17:51'!
identity 	
		
	^#red! !

!RedComponent methodsFor: 'accessing' stamp: 'ShruthiSudhanva 7/17/2013 21:05'!
color
	^Color red.! !

EverettComponent subclass: #ScoreComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!ScoreComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/24/2013 19:41'!
identity
	^#score! !

EverettComponent subclass: #VelocityComponent
	instanceVariableNames: 'velocity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!VelocityComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the velocity of an entity.!

!VelocityComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:27'!
identity 	
	^#velocity! !

!VelocityComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
velocity: aVelocity

	^(self new velocity: aVelocity)! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity

	^velocity ! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity: aVelocity
	
	velocity := aVelocity! !

Object subclass: #EverettGame
	instanceVariableNames: 'canvas prevUpdateTime em sm gm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettGame commentStamp: '<historical>' prior: 0!
Abstract game class that holds canvas, entity manager and system manager. The main game loop should be placed in this class.!

!EverettGame methodsFor: 'gameplay' stamp: 'eshkrm 7/22/2013 16:17'!
calculateDeltaTime

	| currUpdateTime prevMilliseconds currMilliseconds secondsDt millisecondsDt |
	
	currUpdateTime := Time now.
	
	prevMilliseconds := (prevUpdateTime nanoSecond) / 1000000000.
	currMilliseconds := (currUpdateTime nanoSecond) / 1000000000.
	(currMilliseconds < prevMilliseconds) ifTrue: [ currMilliseconds := currMilliseconds + 1 ].
	
	millisecondsDt := currMilliseconds - prevMilliseconds.
	secondsDt := (currUpdateTime subtractTime: prevUpdateTime) seconds.
	
	prevUpdateTime := currUpdateTime.
	
	^(secondsDt + millisecondsDt)! !

!EverettGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:48'!
initialize

	super initialize.
	
	em := EntityManager new.
	sm := SystemManager new.
	gm := em groupManager.
	self initializeCanvas.
	self initializeGroups.
	self initializeEntities.
	self initializeSystems.
	! !

!EverettGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:22'!
initializeCanvas

	^self subclassResponsibility ! !

!EverettGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:21'!
initializeEntities

	^self subclassResponsibility ! !

!EverettGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:34'!
initializeGroups

	^self subclassResponsibility! !

!EverettGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:21'!
initializeSystems

	^self subclassResponsibility ! !

!EverettGame methodsFor: 'gameplay' stamp: 'eshkrm 7/22/2013 16:34'!
play

	^self subclassResponsibility ! !

EverettGame subclass: #PongGame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 19:42'!
initializeCanvas

	canvas := BorderedMorph new.
	canvas color: (Color white).
	canvas bounds: (0 @ 0 extent: (512) @ (512) + (2 * canvas borderWidth)).! !

!PongGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 19:53'!
initializeEntities

	| bat1 bat2 ball mc1 mc2 mc3 |
		
	bat1 := em createEntity.
	bat2 := em createEntity.	
	ball := em createEntity.
	
	gm addEntities: { bat1. bat2 } toGroup: #bats.
	gm addEntities: { ball } toGroup: #ball.
		
	mc1 := BatMorphComponent addMorphToCanvas: canvas.
	mc2 := BatMorphComponent addMorphToCanvas: canvas.
	mc3 := BallMorphComponent addMorphToCanvas: canvas.
	mc1 position: 45@60.
	mc2 position: 467@60.
	mc3 position: 256@256.
	
	em addComponent: mc1 toEntity: bat1.
	em addComponent: mc2 toEntity: bat2.
	em addComponent: mc3 toEntity: ball.
	
	em addComponent: (VelocityComponent velocity: 0@0) toEntity: bat1.
	em addComponent: (VelocityComponent velocity: 0@0) toEntity: bat2.
	em addComponent: (VelocityComponent velocity: 0@0) toEntity: ball! !

!PongGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 18:55'!
initializeGroups

	gm createGroup: #bats.
	gm createGroup: #ball! !

!PongGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 19:41'!
initializeSystems

	sm addSystem: (PongMovementSystem new).! !

!PongGame methodsFor: 'gameplay' stamp: 'eshkrm 7/24/2013 19:46'!
play

	| dt |
	
	canvas openInWorld.
	prevUpdateTime := Time now.
	[ [ true ] whileTrue: [ 
		dt := self calculateDeltaTime. sm updateEntitiesOf: em afterDeltaTime: dt ] ] forkAt: (Processor activePriority - 1) ! !

EverettGame subclass: #TicTacToeGame
	instanceVariableNames: 'canvasIsOpen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!TicTacToeGame methodsFor: 'private' stamp: 'eshkrm 7/23/2013 15:37'!
addEntity: anEntity toRowGroup: i toColumnGroup: j

	gm addEntity: anEntity toGroup: ('row', i asString);
	addEntity: anEntity toGroup: ('col', j asString ) asSymbol.
	
	(i = j) 
	ifTrue: [ gm addEntity: anEntity toGroup: #diag1].
	
	(((i - j) abs = 2) | (i = 2 & j =2))
	ifTrue: [ gm addEntity: anEntity toGroup: #diag2 ]
			! !

!TicTacToeGame methodsFor: 'private' stamp: 'ShruthiSudhanva 7/24/2013 00:13'!
canvasIsOpen

	"^World allMorphs includes: canvas."
	^canvasIsOpen.! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:51'!
initialize

	super initialize! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:48'!
initializeCanvas

	canvas := BorderedMorph new.
	canvas bounds: (5 @ 5 extent: (64 * 3) @ (64 * 3) + (2 * canvas borderWidth))! !

!TicTacToeGame methodsFor: 'private' stamp: 'eshkrm 7/23/2013 16:09'!
initializeCellAt: i at: j

	"Create a cell for position (i,j) and add it to my on-screen representation at the appropriate screen position."
	| indexComponent morphComponent origin e |
	
	morphComponent := CellMorphComponent new.
	canvas addMorph: (morphComponent morph).
	origin := canvas innerBounds origin.
	morphComponent position: ( (i - 1) * morphComponent width) @ ((j - 1) * morphComponent height ) + origin.

	indexComponent := BoardIndexComponent new.
	indexComponent boardIndex: i@j.
	
 	e := em createEntity.
	em addComponent: morphComponent toEntity: e.
	em addComponent: indexComponent toEntity: e.
	
	self addEntity: e toRowGroup: i toColumnGroup: j! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:55'!
initializeEntities 

	1 to: 3 do: [ :i | 1 to: 3 do: [ :j | self initializeCellAt: i at: j ] ]! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:44'!
initializeGroups
	
	1 to: 3 do: [ :i | gm createGroup: (('row',i asString ) asSymbol) ].
	1 to: 3 do: [ :j | gm createGroup: (('col',j asString ) asSymbol) ].
	
	gm createGroup: #diag1; createGroup: #diag2! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/24/2013 19:47'!
initializeSystems

	| victorySystem |
	
	sm addSystem: (InputHandlerSystem onCanvas: canvas).
	sm addSystem: ( victorySystem := VictorySystem onCanvas: canvas ).

	victorySystem addDependent: self.! !

!TicTacToeGame methodsFor: 'gameplay' stamp: 'ShruthiSudhanva 7/24/2013 00:14'!
play

	canvasIsOpen := true.
	canvas openInWorld.
	
	[ [self canvasIsOpen] whileTrue: [ sm updateEntitiesOf: em afterDeltaTime: 1 ] ] forkAt: (Processor activePriority - 1) 
	
	! !

!TicTacToeGame methodsFor: 'private' stamp: 'ShruthiSudhanva 7/24/2013 00:14'!
update: aSymbol
	
	canvasIsOpen := false.
	(Delay forSeconds: 3) wait.
	canvas delete.
	self initialize.
	self play.
	! !

Object subclass: #EverettSystem
	instanceVariableNames: 'canvas updateInterval lastProcessed interestedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettSystem commentStamp: '<historical>' prior: 0!
I am a system that processes entities.
!

!EverettSystem class methodsFor: 'instance creation' stamp: 'eshkrm 7/14/2013 15:20'!
identity

	^self subclassResponsibility ! !

!EverettSystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/22/2013 16:09'!
onCanvas: aCanvas

	^(self new onCanvas: aCanvas)! !

!EverettSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/14/2013 15:10'!
initialize

	updateInterval := 0.
	lastProcessed := 0.
	interestedEntities := Set new! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 15:17'!
interestedEntities

	^interestedEntities! !

!EverettSystem methodsFor: 'accessing' stamp: 'eshkrm 7/11/2013 19:52'!
lastProcessed 

	^lastProcessed! !

!EverettSystem methodsFor: 'accessing' stamp: 'eshkrm 7/11/2013 18:46'!
lastProcessed: aTime

	lastProcessed := aTime! !

!EverettSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/22/2013 16:07'!
onCanvas: aCanvas

	canvas := aCanvas! !

!EverettSystem methodsFor: 'system-processing' stamp: 'eshkrm 7/14/2013 15:11'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt
	
	interestedEntities do: [ :entity | self updateEntity: entity from: anEntityManager afterDeltaTime: dt ].
	! !

!EverettSystem methodsFor: 'system-processing' stamp: 'EricShahkarami 7/5/2013 11:12'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	^self subclassResponsibility! !

!EverettSystem methodsFor: 'accessing' stamp: 'eshkrm 7/11/2013 19:52'!
updateInterval

	^updateInterval! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 15:59'!
updateInterval: anUpdateInterval
	
	updateInterval := anUpdateInterval ! !

!EverettSystem methodsFor: 'testing' stamp: 'eshkrm 7/14/2013 15:20'!
verifyEntity: anEntity from: anEntityManager

	^self subclassResponsibility
! !

EverettSystem subclass: #InputHandlerSystem
	instanceVariableNames: 'turn newlyCapturedComponent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!InputHandlerSystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/23/2013 17:51'!
identity

	^#inputHandler! !

!InputHandlerSystem methodsFor: 'private' stamp: 'eshkrm 7/23/2013 16:25'!
changeTurns

	(turn = #red) ifTrue: [ turn := #blue ] ifFalse: [ turn := #red ].! !

!InputHandlerSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/23/2013 16:13'!
initialize
	
	super initialize.
	turn := #red.! !

!InputHandlerSystem methodsFor: 'system-processing' stamp: 'eshkrm 7/23/2013 16:21'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt
	
	| clickedCell |

	clickedCell := interestedEntities detect: [ :e | 
		(anEntityManager component: (CellMorphComponent identity) fromEntity: e) wasClicked = true ] ifNone: [ nil ].
	clickedCell ifNotNil: [ self updateEntity: clickedCell from: anEntityManager afterDeltaTime: dt. self changeTurns ].! !

!InputHandlerSystem methodsFor: 'system-processing' stamp: 'eshkrm 7/23/2013 16:25'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	| morphComponent colorComponent |
		
	(turn = #red) 
		ifTrue: [ colorComponent := RedComponent new ] 
		ifFalse: [ colorComponent := BlueComponent new ].
	anEntityManager addComponent: colorComponent toEntity: anEntity.
	
	morphComponent := anEntityManager component: (CellMorphComponent identity) fromEntity: anEntity.
	morphComponent color: (colorComponent color).
	
	anEntityManager addComponent: (NewlyCapturedComponent new) toEntity: anEntity.! !

!InputHandlerSystem methodsFor: 'testing' stamp: 'eshkrm 7/23/2013 16:22'!
verifyEntity: anEntity from: anEntityManager 

	| containsCellMorphComponent containsRedComponent containsBlueComponent |
	
	containsCellMorphComponent := anEntityManager doesEntity: anEntity containComponent: CellMorphComponent identity.
	containsRedComponent := anEntityManager doesEntity: anEntity containComponent: RedComponent identity.
	containsBlueComponent := anEntityManager doesEntity: anEntity containComponent: BlueComponent identity.
	
	^(containsCellMorphComponent & containsRedComponent not & containsBlueComponent not).! !

EverettSystem subclass: #MovementSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!MovementSystem commentStamp: '<historical>' prior: 0!
I am a system that updates the position of entities.!

!MovementSystem class methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:55'!
identity 

	^#movement! !

!MovementSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 15:56'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| pc vc |
	
	pc := anEntityManager component: (PositionComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	pc position: ( pc position + (dt * (vc velocity)) )! !

!MovementSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/14/2013 15:20'!
verifyEntity: anEntity from: anEntityManager

	| interestedComponents |
	
	interestedComponents := { PositionComponent identity. VelocityComponent identity. } asSet.
	^(anEntityManager doesEntity: anEntity containComponents: interestedComponents) ! !

EverettSystem subclass: #PongCollisionSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongCollisionSystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/18/2013 13:25'!
identity

	^#collision! !

!PongCollisionSystem methodsFor: 'private' stamp: 'eshkrm 7/19/2013 13:50'!
doesBallCollideWithLeftBat: aBat from: anEntityManager

	| gm ball mc1 mc2 isCollision |
	
	gm := anEntityManager groupManager.
	ball := ((gm entitiesInGroup: #ball) asArray) first.
	
	mc1 := anEntityManager component: (BatMorphComponent identity) fromEntity: aBat.
	mc2 := anEntityManager component: (BallMorphComponent identity) fromEntity: ball.
	
	isCollision := (mc2 left < mc1 right) & (mc2 bottom < mc1 top) & (mc2 top > mc1 bottom).
	^isCollision! !

!PongCollisionSystem methodsFor: 'private' stamp: 'eshkrm 7/19/2013 13:50'!
doesBallCollideWithRightBat: aBat from: anEntityManager

	| gm ball mc1 mc2 isCollision |
	
	gm := anEntityManager groupManager.
	ball := ((gm entitiesInGroup: #ball) asArray) first.
	
	mc1 := anEntityManager component: (BatMorphComponent identity) fromEntity: aBat.
	mc2 := anEntityManager component: (BallMorphComponent identity) fromEntity: ball.
		
	isCollision := (mc2 right < mc1 left) & (mc2 bottom < mc1 top) & (mc2 top > mc1 bottom).
	^isCollision! !

!PongCollisionSystem methodsFor: 'private' stamp: 'eshkrm 7/19/2013 12:55'!
handleCollisionOfBallWithBat: aBat from: anEntityManager! !

!PongCollisionSystem methodsFor: 'processing' stamp: 'eshkrm 7/19/2013 12:44'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	| gm |
	
	gm := anEntityManager groupManager.
	! !

!PongCollisionSystem methodsFor: 'testing' stamp: 'eshkrm 7/24/2013 19:51'!
verifyEntity: anEntity from: anEntityManager

	| isValidEntity |
	
	isValidEntity := anEntityManager doesEntity: anEntity containComponent: (BatMorphComponent identity).
	isValidEntity := isValidEntity or: (anEntityManager doesEntity: anEntity containComponent: BallMorphComponent identity).
		
	^isValidEntity! !

EverettSystem subclass: #PongMovementSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongMovementSystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/17/2013 16:22'!
identity 

	#movement! !

!PongMovementSystem methodsFor: 'processing' stamp: 'eshkrm 7/18/2013 21:13'!
updateBallEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| mc vc |
		
	mc := anEntityManager component: (BallMorphComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	mc position: ( mc position + (dt * (vc velocity)) ).! !

!PongMovementSystem methodsFor: 'processing' stamp: 'eshkrm 7/24/2013 19:44'!
updateBatEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| mc vc |
		
	mc := anEntityManager component: (BatMorphComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	mc position: ( mc position + (dt * (vc velocity)) ).
	vc velocity: (self updateBatVelocity: (vc velocity))
	 ! !

!PongMovementSystem methodsFor: 'private' stamp: 'eshkrm 7/24/2013 19:45'!
updateBatVelocity: aVelocity

	| norm |
	
	norm := (aVelocity dist: 0@0) abs.
	(norm < 1) ifTrue: [ ^(0@0) ] ifFalse: [ ^(0.75 * aVelocity) ]  ! !

!PongMovementSystem methodsFor: 'processing' stamp: 'eshkrm 7/18/2013 21:10'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	| gm |

	gm := anEntityManager groupManager.
	gm entitiesInGroup: #bats do: [ :entity | self updateBatEntity: entity from: anEntityManager afterDeltaTime: dt ].
	gm entitiesInGroup: #ball do: [ :entity | self updateBallEntity: entity from: anEntityManager afterDeltaTime: dt ].! !

!PongMovementSystem methodsFor: 'testing' stamp: 'eshkrm 6/13/2013 19:43'!
verifyEntity: anEntity from: anEntityManager

	| isValidEntity |
	
	isValidEntity := anEntityManager doesEntity: anEntity containComponent: (BatMorphComponent identity).
	isValidEntity := isValidEntity or: (anEntityManager doesEntity: anEntity containComponent: BallMorphComponent identity).
	isValidEntity := isValidEntity and: (anEntityManager doesEntity: anEntity containComponent: VelocityComponent identity).
		
	^isValidEntity
	
! !

EverettSystem subclass: #VictorySystem
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!VictorySystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/23/2013 17:51'!
identity 

	^#victory! !

!VictorySystem methodsFor: 'private' stamp: 'eshkrm 7/23/2013 17:29'!
checkGroup: aGroup forStrikeIn: anEntityManager

	| gm |
	
	gm := anEntityManager groupManager.
	(gm doesGroup: aGroup containComponent: (RedComponent identity)) ifTrue: [ ^true ] .
	(gm doesGroup: aGroup containComponent: (BlueComponent identity)) ifTrue: [ ^true ].
	^false
	
	! !

!VictorySystem methodsFor: 'private' stamp: 'eshkrm 7/23/2013 17:21'!
checkStrikeIn: anEntityManager

	| groups |

	groups := #(#row1 #row2 #row3 #col1 #col2 #col3 #diag1 #diag2).
	groups do: [ :group | (self checkGroup: group forStrikeIn: anEntityManager) ifTrue: [ ^true ] ].
	^false
	
	
	
	! !

!VictorySystem methodsFor: 'private' stamp: 'eshkrm 7/24/2013 14:14'!
declareWinner: anEntity in: anEntityManager

	| winner dialog nextLine |
	
	(anEntityManager doesEntity: anEntity containComponent: (RedComponent identity))
		ifTrue: [ winner := #red ]
		ifFalse: [ winner := #blue ].

	dialog := AboutDialogWindow new.
	nextLine := ' Creating new game.'.
	(winner = #red) ifTrue: [ dialog title: 'Game Won by Player Red!!', nextLine ] ifFalse: [dialog title: 'Game Won by Player Blue!!', nextLine]. 
	dialog openInWorld.
	
	self reset.
	
	
	
	
	
	
	
	
	! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/23/2013 23:44'!
reset

	self changed: #reset.! !

!VictorySystem methodsFor: 'processing' stamp: 'ShruthiSudhanva 7/23/2013 23:02'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	(self checkStrikeIn: anEntityManager) ifTrue: [ self declareWinner: anEntity in: anEntityManager ].
	anEntityManager removeComponent: (NewlyCapturedComponent identity) fromEntity: anEntity.
! !

!VictorySystem methodsFor: 'testing' stamp: 'eshkrm 7/23/2013 16:52'!
verifyEntity: anEntity from: anEntityManager 
		
	^(anEntityManager doesEntity: anEntity containComponent: (NewlyCapturedComponent identity))
	
	! !

Object subclass: #GroupManager
	instanceVariableNames: 'groups entityManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!GroupManager commentStamp: '<historical>' prior: 0!
I manage groups of entities, each identified with a tag.!

!GroupManager class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/14/2013 15:21'!
entityManager: anEntityManager

	^self new entityManager: anEntityManager.! !

!GroupManager methodsFor: 'group-assignment' stamp: 'eshkrm 7/14/2013 14:53'!
activateEntitiesInGroup: aGroupIdentifier

	self entitiesInGroup: aGroupIdentifier do: [ :entity | entityManager activateEntity: entity ]! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/14/2013 14:51'!
addComponent: aComponentInstance toGroup: aGroupIdentifier

	(self entitiesInGroup: aGroupIdentifier) do: [ :entity | entityManager addComponent: aComponentInstance toEntity: entity ]! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/24/2013 19:38'!
addEntities: aCollectionOfEntities toGroup: aGroup

	aCollectionOfEntities do: [ :entity | self addEntity: entity toGroup: aGroup ]! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/8/2013 23:00'!
addEntity: anEntity toGroup: aGroupIdentifier

	(groups at: aGroupIdentifier) add: anEntity! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/8/2013 22:55'!
createGroup: aGroupIdentifier

	groups at: aGroupIdentifier put: Set new! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/14/2013 14:53'!
createGroup: aGroupIdentifier withEntities: aCollectionOfEntities

	groups at: aGroupIdentifier put: (aCollectionOfEntities asSet)! !

!GroupManager methodsFor: 'group-assignment' stamp: 'eshkrm 7/14/2013 14:54'!
deactivateEntitiesInGroup: aGroupIdentifier

	self entitiesInGroup: aGroupIdentifier do: [ :entity | entityManager deactivateEntity: entity ]! !

!GroupManager methodsFor: 'testing' stamp: 'eshkrm 7/23/2013 17:39'!
doesGroup: aGroup containComponent: aComponentIdentifier
	
	| detectedEntity |
	
	detectedEntity := self entitiesInGroup: aGroup detect: 
		[ :entity | (entityManager doesEntity: entity containComponent: aComponentIdentifier) = false ] ifNone: [ -1 ].
	
	^(detectedEntity < 0) 
	! !

!GroupManager methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 20:57'!
doesGroup: aGroup includeEntity: anEntity

	^( (self entitiesInGroup: aGroup) includes: anEntity )! !

!GroupManager methodsFor: 'accessing' stamp: 'eshkrm 7/14/2013 14:57'!
entitiesInGroup: aGroupIdentifier

	^(groups at: aGroupIdentifier)! !

!GroupManager methodsFor: 'group-enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier collect: aBlock

	^( (groups at: aGroupIdentifier) collect: [ :entity | aBlock value: entity ] )! !

!GroupManager methodsFor: 'group-enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier detect: aBlock

	^( (groups at: aGroupIdentifier) detect: [ :entity | aBlock value: entity ] ).	! !

!GroupManager methodsFor: 'group-enumerate' stamp: 'eshkrm 7/23/2013 17:37'!
entitiesInGroup: aGroupIdentifier detect: aBlock ifNone: anotherBlock

	^( (groups at: aGroupIdentifier) detect: [ :entity | aBlock value: entity ] ifNone: anotherBlock ).	! !

!GroupManager methodsFor: 'group-enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier do: aBlock

	(groups at: aGroupIdentifier) do: [ :entity | aBlock value: entity ]! !

!GroupManager methodsFor: 'group-enumerate' stamp: 'eshkrm 7/8/2013 23:02'!
entitiesInGroup: aGroupIdentifier select: aBlock

	^( (groups at: aGroupIdentifier) select: [ :entity | aBlock value: entity ] )! !

!GroupManager methodsFor: 'initialize-release' stamp: 'ShruthiSudhanva 7/12/2013 12:11'!
entityManager: anEntityManager 	

	super initialize.
	groups := Dictionary new.
	entityManager := anEntityManager.! !

!GroupManager methodsFor: 'accessing' stamp: 'eshkrm 7/14/2013 14:51'!
groups

	^(groups keys).! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/14/2013 14:51'!
removeComponent: aComponentIdentifier fromGroup: aGroupIdentifier

	(self entitiesInGroup: aGroupIdentifier) do: [ :entity | entityManager removeComponent: aComponentIdentifier fromEntity: entity ]
! !

!GroupManager methodsFor: 'add-remove' stamp: 'ShruthiSudhanva 7/7/2013 20:15'!
removeEntity: anEntity fromGroup: aGroupIdentfier

	(groups at: aGroupIdentfier) remove: anEntity. 	! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/8/2013 22:55'!
removeGroup: aGroupIdentifier

	groups removeKey: aGroupIdentifier
! !

Object subclass: #SystemManager
	instanceVariableNames: 'priorities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!SystemManager commentStamp: '<historical>' prior: 0!
I manage all running systems.!

!SystemManager methodsFor: 'add-remove' stamp: 'EricShahkarami 7/1/2013 15:18'!
addSystem: aSystemInstance 
	
	self addSystem: aSystemInstance withPriority: 0 ! !

!SystemManager methodsFor: 'add-remove' stamp: 'eshkrm 7/7/2013 21:01'!
addSystem: aSystemInstance withPriority: aPriority
	
	self priorityAt: aPriority putSystem: aSystemInstance.


	! !

!SystemManager methodsFor: 'system-processing' stamp: 'eshkrm 7/11/2013 18:56'!
haveSystem: system updateEntitiesFrom: anEntityManager afterDeltaTime: dt

	| lastProcessed updateInterval |
	
	self resolveChangedEntitiesFrom: anEntityManager withSystem: system.
	self resolveInactiveEntitiesFrom: anEntityManager withSystem: system.	
	self resolveRemovedEntitiesFrom: anEntityManager withSystem: system.
		
	lastProcessed := (system lastProcessed) + dt.
	updateInterval := system updateInterval.
	
	(lastProcessed >= updateInterval) ifTrue: 
	[ 
		system updateEntitiesOf: anEntityManager afterDeltaTime: dt.
		system lastProcessed: 0.
	]
	ifFalse:
	[  
		system lastProcessed: lastProcessed.
	]! !

!SystemManager methodsFor: 'initialize-release' stamp: 'eshkrm 7/8/2013 22:39'!
initialize
	
	super initialize.
	priorities := SortedCollection sortBlock: [ :p1 :p2 | p1 key >= p2 key ]! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/7/2013 20:16'!
priorityAt: aPriority

	| systemsWithAPriority |
	
	systemsWithAPriority := priorities detect: [ :assoc | assoc key = aPriority ].
	^(systemsWithAPriority value)! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/7/2013 21:00'!
priorityAt: aPriority putSystem: aSystemInstance

	| systemsWithAPriority |
	
	(priorities includes: aPriority) ifFalse: [ priorities add: ( aPriority->(Dictionary new) ) ].
	
	systemsWithAPriority := self priorityAt: aPriority.
	systemsWithAPriority at: (aSystemInstance class identity) put: aSystemInstance! !

!SystemManager methodsFor: 'add-remove' stamp: 'ShruthiSudhanva 7/7/2013 22:02'!
removeSystem: aSystemIdentifier

	| systems |

	systems := self systemsWithPriorityDetect: [ :systemsWithPriority | systemsWithPriority includesKey: aSystemIdentifier ].
	systems ifNotNil: [ systems removeKey: aSystemIdentifier ]
	! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/14/2013 15:09'!
resolveChangedEntitiesFrom: anEntityManager withSystem: aSystem

	| interestedEntities |
	
	interestedEntities := aSystem interestedEntities.

	anEntityManager changedEntitiesDo: [ :entity | 
		(aSystem verifyEntity: entity from: anEntityManager) 
			ifTrue: [ (interestedEntities includes: entity) ifFalse: [ interestedEntities add: entity ]. ]
			ifFalse: [ (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ].
	]! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/11/2013 18:58'!
resolveInactiveEntitiesFrom: anEntityManager withSystem: aSystem

	| interestedEntities |
	
	interestedEntities := aSystem interestedEntities.
	anEntityManager inactiveEntitiesDo: [ :entity | (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ]! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/11/2013 18:59'!
resolveRemovedEntitiesFrom: anEntityManager withSystem: aSystem 

	| interestedEntities |
	
	interestedEntities := aSystem interestedEntities.
	anEntityManager removedEntitiesDo: [ :entity | (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ]! !

!SystemManager methodsFor: 'system-enumerate' stamp: 'eshkrm 7/7/2013 21:23'!
systemsDo: aBlock

	"Run block on systems in order of their priority"
	priorities do: [ :assoc | (assoc value) do: [ :system | aBlock value: system ] ]! !

!SystemManager methodsFor: 'system-enumerate' stamp: 'eshkrm 7/7/2013 21:14'!
systemsWithPriorityDetect: aBlock

	| systemsWithPriority |

	systemsWithPriority := priorities detect: [ :assoc | aBlock value: (assoc value) ].
	^systemsWithPriority ! !

!SystemManager methodsFor: 'system-enumerate' stamp: 'eshkrm 7/7/2013 21:23'!
systemsWithPriorityDo: aBlock 
	
	"Run block on systems with same priority."
	priorities do: [ :assoc | aBlock value: (assoc value ) ]! !

!SystemManager methodsFor: 'system-processing' stamp: 'eshkrm 7/11/2013 18:47'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	self systemsDo: [ :system | self haveSystem: system updateEntitiesFrom: anEntityManager afterDeltaTime: dt ].
	
	anEntityManager clearChangedEntities.
	anEntityManager clearRemovedEntities.
	! !
